@@ -1,20 +1,27 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+# Created on Wed Nov  6 14:33:36 2019
+
 """
--------------------------------------------------------------------------------
+
 spatial_analysis
-functions to analyse data in raster and/or feature datasets
-to replace ArcGridDataProcessing
--------------------------------------------------------------------------------
-@author: Xiaodong Ming
-Created on Wed Nov  6 14:33:36 2019
--------------------------------------------------------------------------------
+================
+
+functions to analyse data in raster and/or feature datasets to replace ArcGridDataProcessing
+
+
 Assumptions:
-- map unit is meter
-- its cellsize is the same in both x and y direction
-- its reference position is on the lower left corner of the southwest cell
+    - map unit is meter
+
+    - its cellsize is the same in both x and y direction
+
+    - its reference position is on the lower left corner of the southwest cell
+
 To do:
-- read and write arc
+    - read and write arc
+
+-----------------
+
 """
 __author__ = "Xiaodong Ming"
 import os
@@ -30,18 +37,27 @@ from mpl_toolkits.axes_grid1 import make_axes_locatable
 #%%
 def arc_header_read(file_name, header_rows=6):
     """ read the header of a asc file as a dictionary
+
     file_name: (string) file name
+
     header_rows: (int) number of header rows
+
     Return:
+
         header: a dictionary with keys:
+
             ncols: (int) number of columns
+
             nrows: (int) number of rows
-            xllcorner: (int/float) x-coordinate of the lower left corner of
-                the lower left cell of the grid
-            yllcorner: (int/float) y-coordinate of the lower left corner of
-                the bottom left cell of the grid
+
+            xllcorner: (int/float) x-coordinate of the lower left corner of the lower left cell of the grid
+
+            yllcorner: (int/float) y-coordinate of the lower left corner of the bottom left cell of the grid
+
             cellsize: (int/float) the length of one square cell
+
             NODATA_value: (int/float)|-9999 the value representing nodata cell
+
     """
     check_file_existence(file_name)
     # read header
@@ -72,13 +88,21 @@ def arc_header_read(file_name, header_rows=6):
 
 def arcgridread(file_name, header_rows=6, return_nan=True):
     """ Read ArcGrid format raster file
+
     file_name: (str) the file name to read data
+
     header_rows: (int) the number of head rows of the asc file
+
     Return:
+
         array: (int/float numpy array)
+
         header: (dict) to provide reference information of the grid
+
         extent: (tuple) outline extent of the grid (left, right, bottom, top)
+
     Note: this function can also read compressed gz files
+
     """
     check_file_existence(file_name)
     # read header
@@ -98,12 +122,19 @@ def arcgridread(file_name, header_rows=6, return_nan=True):
 
 def arcgridwrite(file_name, array, header, compression=False):
     """ write gird data into a ascii file
-    file_name: (str) the file name to write grid data. A compressed file will
-        automatically add a suffix '.gz'
+
+    file_name: (str) the file name to write grid data. A compressed file will automatically add a suffix '.gz'
+
     array: (int/float numpy array)
+
     header: (dict) to provide reference information of the grid
+
     compression: (logic) to inidcate whether compress the ascii file
+
     Example:
+
+    .. code:: python
+
         gird = np.zeros((5,10))
         grid[0,:] = -9999
         grid[-1,:] = -9999
@@ -112,6 +143,7 @@ def arcgridwrite(file_name, array, header, compression=False):
         file_name = 'example_file.asc'
         arcgridwrite(file_name, array, header, compression=False)
         arcgridwrite(file_name, array, header, compression=True)
+
     """
     array = array+0
     if not isinstance(header, dict):
@@ -138,8 +170,7 @@ def arcgridwrite(file_name, array, header, compression=False):
 
 def tif_read(file_name):
     """
-    read tif file and return array, header, projection
-    only read the first band
+    read tif file and return array, header, projection only read the first band
     """
     from osgeo import gdal
     ds = gdal.Open(file_name)        
@@ -186,8 +217,10 @@ def byte_file_read(file_name):
 #%% Combine raster files
 def combine_raster(asc_files, num_header_rows=6):
     """Combine a list of asc files to a DEM Raster
+
     asc_files: a list of asc file names
-    all raster files have the same cellsize
+        
+        all raster files have the same cellsize
     """
     # default values for the combined Raster file
     xllcorner_all = []
@@ -245,10 +278,13 @@ def map_show(array, header, figname=None, figsize=None, dpi=300,
              cax=True, relocate=False, scale_ratio=1):
     """
     Display raster data
-    figname: the file name to export map, if figname is empty, then
-        the figure will not be saved
+
+    figname: the file name to export map, if figname is empty, then the figure will not be saved
+
     figsize: the size of map
+
     dpi: The resolution in dots per inch
+
     vmin and vmax define the data range that the colormap covers
     """
     np.warnings.filterwarnings('ignore')
@@ -278,8 +314,8 @@ def rank_show(array, header, figname=None, figsize=None, dpi=300,
             show_colorbar=True, show_colorlegend=False,
             relocate=False, scale_ratio=1):
     """ 
-    Categorize array data as ranks according to the breaks and display a ranked
-        map
+    Categorize array data as ranks according to the breaks and display a ranked map
+
     """
     np.warnings.filterwarnings('ignore')
     array = array+0
@@ -340,8 +376,7 @@ def check_file_existence(file_name):
         raise
 
 def header2extent(header):
-    """ convert a header dict to a 4-element tuple (left, right, bottom, top)
-    all four elements shows the coordinates at the edge of a cell, not center
+    """ convert a header dict to a 4-element tuple (left, right, bottom, top) all four elements shows the coordinates at the edge of a cell, not center
     """
     left = header['xllcorner']
     right = header['xllcorner']+header['ncols']*header['cellsize']
@@ -369,8 +404,11 @@ def shape_extent_to_header(shape, extent, nan_value=-9999):
 
 def map2sub(X, Y, header):
     """ convert map coordinates to subscripts of an array
-    array is defined by a geo-reference header
+
+        array is defined by a geo-reference header
+
     X, Y: a scalar or numpy array of coordinate values
+
     Return: rows, cols in the array
     """
     # X and Y coordinate of the centre of the first cell in the array
@@ -390,8 +428,8 @@ def map2sub(X, Y, header):
 
 def sub2map(rows, cols, header):
     """
-    convert subscripts of a matrix to map coordinates 
-    rows, cols: subscripts of the data matrix, starting from 0
+    convert subscripts of a matrix to map coordinates rows, cols: subscripts of the data matrix, starting from 0
+
     return
         X, Y: coordinates in map units
     """
@@ -409,12 +447,19 @@ def sub2map(rows, cols, header):
 #% Extent compare between two Raster objects
 def compare_extent(extent0, extent1):
     """Compare and show the difference between two Raster extents
+
     extent0, extent1: objects or extent dicts to be compared
+
     displaye: whether to show the extent in figures
+
     Return:
+
         0 extent0>=extent1
+
         1 extent0<extent1
+
         2 extent0 and extent1 have intersections
+
     """
     logic_left = extent0[0]<=extent1[0]
     logic_right = extent0[1]>=extent1[1]
@@ -443,11 +488,10 @@ def extent2shape_points(extent):
 
 def _adjust_map_extent(extent, relocate=True, scale_ratio=1):
     """
-    Adjust the extent (left, right, bottom, top) to a new staring point 
-        and new unit. extent values will be divided by the scale_ratio
+    Adjust the extent (left, right, bottom, top) to a new staring point and new unit. extent values will be divided by the scale_ratio
+
     Example:
-        if scale_ratio = 1000, and the original extent unit is meter,
-        then the unit is converted to km, and the extent is divided by 1000
+        if scale_ratio = 1000, and the original extent unit is meter, then the unit is converted to km, and the extent is divided by 1000
     """
     if relocate:
         left = 0 
@@ -501,4 +545,4 @@ def main():
     print('Fucntions to process asc data')
 
 if __name__=='__main__':
-    main()
\ No newline at end of file
+    main()
